-- by modelleicher
-- www.schwabenmodding.de
-- date: 22.07.2017
-- playerControlledAnimation animation 

-- animationName -> name of the animation
-- speed -> speed the animation will be played at
-- sharedAnimation -> set to true if animation will also be played from other scripts. That way the script will check position of animation first so you don't have to press key twice
-- l10nName -> optional, if left out l10n Text name is "togglePlayerControlledAnimation" 

-- <playerControlledAnimation animationName="" speed="" startState="" triggerIndex="" sharedAnimation="" l10nName="" />

playerControlledAnimation = {};

function playerControlledAnimation.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations);
end;

function playerControlledAnimation:load(savegame)
	self.setPlayerControlledAnimation = playerControlledAnimation.setPlayerControlledAnimation;
	self.playerControlledAnimationTrigger = playerControlledAnimation.playerControlledAnimationTrigger;
	
	self.pca = {};
	self.pca.animationName = getXMLString(self.xmlFile, "vehicle.playerControlledAnimation#animationName");
	self.pca.animationSpeed = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.playerControlledAnimation#speed"), 1);
	--self.pca.startState = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.playerControlledAnimation#startState"), nil);
	self.pca.trigger = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.playerControlledAnimation#triggerIndex"));
	self.pca.sharedAnimation = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.playerControlledAnimation#sharedAnimation"), false);
	
	self.pca.l10nName = Utils.getNoNil(getXMLString(self.xmlFile, "vehicle.playerControlledAnimation#l10nName"),"togglePlayerControlledAnimation" );

	self.pca.currentState = false; -- always use currentState of the animation not wanted state for less confusion
	
	-- reset animation during load
	if self.pca.sharedAnimation == false then
		self:playAnimation(self.pca.animationName, -self.pca.animationSpeed, self:getAnimationTime(self.pca.animationName), true);
	end;
	
	self.pca.playerInTrigger = false;
	
	addTrigger(self.pca.trigger, "playerControlledAnimationTrigger", self);	

	
end;

function playerControlledAnimation:readStream(streamId, connection)  
	--print("readStream called");
    self:setPlayerControlledAnimation(streamReadBool(streamId), true);  
end;

function playerControlledAnimation:writeStream(streamId, connection)  
	--print("writeStream called");
	if not self.pca.sharedAnimation then
		streamWriteBool(streamId, self.pca.currentState);  
	else -- get the current state of the animation to synchronize it
		local setState = false;
		if self:getAnimationTime(self.pca.animationName) > 0.5 then 
			setState = true;
		end;
		streamWriteBool(streamId, setState); 
	end;
end;

function playerControlledAnimation:playerControlledAnimationTrigger(triggerId, otherId, onEnter, onLeave, onStay)
	-- basic player in trigger code
	if onEnter and g_currentMission.controlPlayer and g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
		self.pca.playerInTrigger = true;
	elseif onLeave and g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
		self.pca.playerInTrigger = false;
	end;
	
end;

function playerControlledAnimation:update(dt)    
	
	if self.pca.playerInTrigger then

		if self.pca.animationName ~= nil then
			
			-- help text
			if g_currentMission.controlPlayer then
				g_currentMission:addHelpButtonText(g_i18n:getText(self.pca.l10nName), InputBinding.TOGGLE_playerControlledAnimation); 
			end;	
		
			-- if the animation is shared with other scripts, find out where the animation is first
			if self.pca.sharedAnimation then 
				local setState = false;
			
				if self:getAnimationTime(self.pca.animationName) > 0.5 then -- if the animation is over half way we want to reverse it
					setState = true;
				end;
			

				if InputBinding.hasEvent(InputBinding.TOGGLE_playerControlledAnimation) then
					self:setPlayerControlledAnimation(not setState);
				end;
			
			else -- otherwise don't find out where the animation is.
				if InputBinding.hasEvent(InputBinding.TOGGLE_playerControlledAnimation) then
					self:setPlayerControlledAnimation(not self.pca.currentState);
				end;			
			end;
		end;
	end;

end;
function playerControlledAnimation:setPlayerControlledAnimation(wantedState, noEventSend)
	setPlayerControlledAnimationEvent.sendEvent(self, wantedState, noEventSend);
	if wantedState then -- if currentState is true (max) then play animation to min
		self:playAnimation(self.pca.animationName, self.pca.animationSpeed, self:getAnimationTime(self.pca.animationName), true);
		self.pca.currentState = true;
	else	
		self:playAnimation(self.pca.animationName, -self.pca.animationSpeed, self:getAnimationTime(self.pca.animationName), true);
		self.pca.currentState = false;
	end;

end;
function playerControlledAnimation:delete()
	if self.pca.animationName ~= nil then
		removeTrigger(self.pca.trigger);
	end;
end;
function playerControlledAnimation:onLeave()

end;
function playerControlledAnimation:mouseEvent(posX, posY, isDown, isUp, button)
end;
function playerControlledAnimation:keyEvent(unicode, sym, modifier, isDown)
end;

function playerControlledAnimation:draw()
end;


setPlayerControlledAnimationEvent = {};
setPlayerControlledAnimationEvent_mt = Class(setPlayerControlledAnimationEvent, Event);
InitEventClass(setPlayerControlledAnimationEvent, "setPlayerControlledAnimationEvent");

function setPlayerControlledAnimationEvent:emptyNew()  
    local self = Event:new(setPlayerControlledAnimationEvent_mt );
    self.className="setPlayerControlledAnimationEvent";
    return self;
end;
function setPlayerControlledAnimationEvent:new(vehicle, state) 
    self.vehicle = vehicle;
    self.state = state;
    return self;
end;
function setPlayerControlledAnimationEvent:readStream(streamId, connection)  
    local id = streamReadInt32(streamId); 
    self.state = streamReadBool(streamId); 
    self.vehicle = networkGetObject(id); 
    self:run(connection);  
end;
function setPlayerControlledAnimationEvent:writeStream(streamId, connection)   
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));	
    streamWriteBool(streamId, self.state );   
end;
function setPlayerControlledAnimationEvent:run(connection) 
    self.vehicle:setPlayerControlledAnimation(self.state, true);
	if not connection:getIsServer() then  
		g_server:broadcastEvent(setPlayerControlledAnimationEvent:new(self.vehicle, self.state), nil, connection, self.object);
	end;
end;
function setPlayerControlledAnimationEvent.sendEvent(vehicle, state, noEventSend)  
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then   
			g_server:broadcastEvent(setPlayerControlledAnimationEvent:new(vehicle, state), nil, nil, vehicle);
		else 
			g_client:getServerConnection():sendEvent(setPlayerControlledAnimationEvent:new(vehicle, state));
		end;
	end;
end;
